==> v5.6.1-dev.1.tar.gz: revanced-patches-ex/patches/src/main/kotlin/app/revanced/patches/reddit/layout/subredditdialog/SubRedditDialogPatch.kt <==
package app.revanced.patches.reddit.layout.subredditdialog

import app.revanced.patcher.extensions.InstructionExtensions.addInstruction
import app.revanced.patcher.extensions.InstructionExtensions.addInstructions
import app.revanced.patcher.extensions.InstructionExtensions.getInstruction
import app.revanced.patcher.patch.bytecodePatch
import app.revanced.patches.reddit.utils.compatibility.Constants.COMPATIBLE_PACKAGE
import app.revanced.patches.reddit.utils.extension.Constants.PATCHES_PATH
import app.revanced.patches.reddit.utils.patch.PatchList.REMOVE_SUBREDDIT_DIALOG
import app.revanced.patches.reddit.utils.settings.is_2024_41_or_greater
import app.revanced.patches.reddit.utils.settings.is_2025_01_or_greater
import app.revanced.patches.reddit.utils.settings.is_2025_05_or_greater
import app.revanced.patches.reddit.utils.settings.is_2025_06_or_greater
import app.revanced.patches.reddit.utils.settings.settingsPatch
import app.revanced.patches.reddit.utils.settings.updatePatchStatus
import app.revanced.util.fingerprint.methodOrThrow
import app.revanced.util.getReference
import app.revanced.util.indexOfFirstInstructionOrThrow
import app.revanced.util.indexOfFirstInstructionReversedOrThrow
import com.android.tools.smali.dexlib2.Opcode
import com.android.tools.smali.dexlib2.iface.instruction.FiveRegisterInstruction
import com.android.tools.smali.dexlib2.iface.instruction.OneRegisterInstruction
import com.android.tools.smali.dexlib2.iface.reference.MethodReference

private const val EXTENSION_CLASS_DESCRIPTOR =
    "$PATCHES_PATH/RemoveSubRedditDialogPatch;"

@Suppress("unused")
val subRedditDialogPatch = bytecodePatch(
    REMOVE_SUBREDDIT_DIALOG.title,
    REMOVE_SUBREDDIT_DIALOG.summary,
) {
    compatibleWith(COMPATIBLE_PACKAGE)

    dependsOn(settingsPatch)

    execute {

        if (is_2024_41_or_greater) {
            frequentUpdatesHandlerFingerprint
                .methodOrThrow()
                .apply {
                    listOfIsLoggedInInstruction(this)
                        .forEach { index ->
                            val register = getInstruction<OneRegisterInstruction>(index + 1).registerA

                            addInstructions(
                                index + 2, """
                                    invoke-static {v$register}, $EXTENSION_CLASS_DESCRIPTOR->spoofLoggedInStatus(Z)Z
                                    move-result v$register
                                    """
                            )
                        }
                }
        }

        // Not used in latest Reddit client.
        if (!is_2025_05_or_greater) {
            frequentUpdatesSheetScreenFingerprint.methodOrThrow().apply {
                val index = indexOfFirstInstructionReversedOrThrow(Opcode.RETURN_OBJECT)
                val register =
                    getInstruction<OneRegisterInstruction>(index).registerA

                addInstruction(
                    index,
                    "invoke-static {v$register}, $EXTENSION_CLASS_DESCRIPTOR->dismissDialog(Landroid/view/View;)V"
                )
            }
        }

        if (is_2025_01_or_greater) {
            nsfwAlertEmitFingerprint.methodOrThrow().apply {
                val hasBeenVisitedIndex = indexOfHasBeenVisitedInstruction(this)
                val hasBeenVisitedRegister =
                    getInstruction<OneRegisterInstruction>(hasBeenVisitedIndex + 1).registerA

                addInstructions(
                    hasBeenVisitedIndex + 2, """
                        invoke-static {v$hasBeenVisitedRegister}, $EXTENSION_CLASS_DESCRIPTOR->spoofHasBeenVisitedStatus(Z)Z
                        move-result v$hasBeenVisitedRegister
                        """
                )
            }
        }

        // Not used in latest Reddit client.
        if (!is_2025_06_or_greater) {
            redditAlertDialogsFingerprint.methodOrThrow().apply {
                val backgroundTintIndex = indexOfSetBackgroundTintListInstruction(this)
                val insertIndex =
                    indexOfFirstInstructionOrThrow(backgroundTintIndex) {
                        opcode == Opcode.INVOKE_VIRTUAL &&
                                getReference<MethodReference>()?.name == "setTextAppearance"
                    }
                val insertRegister = getInstruction<FiveRegisterInstruction>(insertIndex).registerC

                addInstruction(
                    insertIndex,
                    "invoke-static {v$insertRegister}, $EXTENSION_CLASS_DESCRIPTOR->confirmDialog(Landroid/widget/TextView;)V"
                )
            }
        }

        updatePatchStatus(
            "enableSubRedditDialog",
            REMOVE_SUBREDDIT_DIALOG
        )
    }
}

==> v5.6.2-dev.1.tar.gz: revanced-patches-ex/patches/src/main/kotlin/app/revanced/patches/reddit/layout/subredditdialog/SubRedditDialogPatch.kt <==
package app.revanced.patches.reddit.layout.subredditdialog

import app.revanced.patcher.extensions.InstructionExtensions.addInstruction
import app.revanced.patcher.extensions.InstructionExtensions.addInstructions
import app.revanced.patcher.extensions.InstructionExtensions.addInstructionsWithLabels
import app.revanced.patcher.extensions.InstructionExtensions.getInstruction
import app.revanced.patcher.patch.PatchException
import app.revanced.patcher.patch.bytecodePatch
import app.revanced.patcher.util.smali.ExternalLabel
import app.revanced.patches.reddit.utils.compatibility.Constants.COMPATIBLE_PACKAGE
import app.revanced.patches.reddit.utils.extension.Constants.PATCHES_PATH
import app.revanced.patches.reddit.utils.patch.PatchList.REMOVE_SUBREDDIT_DIALOG
import app.revanced.patches.reddit.utils.resourceid.sharedResourceIdPatch
import app.revanced.patches.reddit.utils.settings.is_2024_41_or_greater
import app.revanced.patches.reddit.utils.settings.is_2025_01_or_greater
import app.revanced.patches.reddit.utils.settings.is_2025_05_or_greater
import app.revanced.patches.reddit.utils.settings.is_2025_06_or_greater
import app.revanced.patches.reddit.utils.settings.settingsPatch
import app.revanced.patches.reddit.utils.settings.updatePatchStatus
import app.revanced.util.findFreeRegister
import app.revanced.util.fingerprint.methodOrThrow
import app.revanced.util.fingerprint.mutableClassOrThrow
import app.revanced.util.getReference
import app.revanced.util.indexOfFirstInstruction
import app.revanced.util.indexOfFirstInstructionOrThrow
import app.revanced.util.indexOfFirstInstructionReversedOrThrow
import com.android.tools.smali.dexlib2.Opcode
import com.android.tools.smali.dexlib2.iface.instruction.FiveRegisterInstruction
import com.android.tools.smali.dexlib2.iface.instruction.OneRegisterInstruction
import com.android.tools.smali.dexlib2.iface.reference.MethodReference

private const val EXTENSION_CLASS_DESCRIPTOR =
    "$PATCHES_PATH/RemoveSubRedditDialogPatch;"

@Suppress("unused")
val subRedditDialogPatch = bytecodePatch(
    REMOVE_SUBREDDIT_DIALOG.title,
    REMOVE_SUBREDDIT_DIALOG.summary,
) {
    compatibleWith(COMPATIBLE_PACKAGE)

    dependsOn(
        settingsPatch,
        sharedResourceIdPatch,
    )

    execute {

        if (is_2024_41_or_greater) {
            frequentUpdatesHandlerFingerprint
                .methodOrThrow()
                .apply {
                    listOfIsLoggedInInstruction(this)
                        .forEach { index ->
                            val register =
                                getInstruction<OneRegisterInstruction>(index + 1).registerA

                            addInstructions(
                                index + 2, """
                                    invoke-static {v$register}, $EXTENSION_CLASS_DESCRIPTOR->spoofLoggedInStatus(Z)Z
                                    move-result v$register
                                    """
                            )
                        }
                }
        }

        // Not used in latest Reddit client.
        if (!is_2025_05_or_greater) {
            frequentUpdatesSheetScreenFingerprint.methodOrThrow().apply {
                val index = indexOfFirstInstructionReversedOrThrow(Opcode.RETURN_OBJECT)
                val register =
                    getInstruction<OneRegisterInstruction>(index).registerA

                addInstruction(
                    index,
                    "invoke-static {v$register}, $EXTENSION_CLASS_DESCRIPTOR->dismissDialog(Landroid/view/View;)V"
                )
            }
        }

        if (is_2025_01_or_greater) {
            arrayOf(
                nsfwAlertEmitFingerprint,
                nsfwAlertObserverFingerprint
            ).forEach { fingerprint ->
                fingerprint.methodOrThrow().apply {
                    val getOverIndex = indexOfGetOverInstruction(this)
                    val getOverRegister = getInstruction<FiveRegisterInstruction>(getOverIndex).registerC
                    val freeRegister = findFreeRegister(getOverIndex, getOverRegister)

                    val returnIndex = indexOfFirstInstructionOrThrow(getOverIndex, Opcode.RETURN_OBJECT)
                    val jumpIndex = indexOfFirstInstructionReversedOrThrow(returnIndex, Opcode.SGET_OBJECT)

                    addInstructionsWithLabels(
                        getOverIndex, """
                            invoke-static {}, $EXTENSION_CLASS_DESCRIPTOR->removeNSFWDialog()Z
                            move-result v$freeRegister
                            if-nez v$freeRegister, :jump
                            """, ExternalLabel("jump", getInstruction(jumpIndex))
                    )
                }
            }

            var hookCount = 0

            nsfwAlertBuilderFingerprint.mutableClassOrThrow().let {
                it.methods.forEach { method ->
                    method.apply {
                        val showIndex = indexOfFirstInstruction {
                            opcode == Opcode.INVOKE_VIRTUAL &&
                                    getReference<MethodReference>()?.name == "show"
                        }
                        if (showIndex >= 0) {
                            val dialogRegister = getInstruction<OneRegisterInstruction>(showIndex + 1).registerA

                            addInstruction(
                                showIndex + 2,
                                "invoke-static {v$dialogRegister}, $EXTENSION_CLASS_DESCRIPTOR->dismissNSFWDialog(Ljava/lang/Object;)V"
                            )
                            hookCount++
                        }
                    }
                }
            }

            if (hookCount == 0) {
                throw PatchException("Failed to find hook method")
            }
        }

        // Not used in latest Reddit client.
        if (!is_2025_06_or_greater) {
            redditAlertDialogsFingerprint.methodOrThrow().apply {
                val backgroundTintIndex = indexOfSetBackgroundTintListInstruction(this)
                val insertIndex =
                    indexOfFirstInstructionOrThrow(backgroundTintIndex) {
                        opcode == Opcode.INVOKE_VIRTUAL &&
                                getReference<MethodReference>()?.name == "setTextAppearance"
                    }
                val insertRegister = getInstruction<FiveRegisterInstruction>(insertIndex).registerC

                addInstruction(
                    insertIndex,
                    "invoke-static {v$insertRegister}, $EXTENSION_CLASS_DESCRIPTOR->confirmDialog(Landroid/widget/TextView;)V"
                )
            }
        }

        updatePatchStatus(
            "enableSubRedditDialog",
            REMOVE_SUBREDDIT_DIALOG
        )
    }
}
